#!/usr/bin/env python
# coding: utf-8

#########################################################################
#########################################################################

"""
   File Name: index.py
      Author: Wan Ji
      E-mail: wanji@live.com
  Created on: Fri Mar 14 17:34:07 2014 CST
"""
DESCRIPTION = """
"""

import logging
import multiprocessing
from itertools import izip
import operator
import array
# import pdb

import numpy as np
import scipy as sp

import scipy.io as spio
from scipy.sparse import csr_matrix
from utils import nsmallest, \
    tic, toc
from distance import distFunc, \
    DotProduct_DML, DotProduct_DML_Diagonal
from feature import FeatLoader
from bitmap import BitMap


class Indexer(object):
    """ Index for retrieval
    """

    def __init__(self, distype, metric=None):
        self.nitems = 0
        self.dim_mask = None
        self.distype = distype
        self.set_metric(metric)

    def __del__(self):
        pass

    @staticmethod
    def load_metric(metric_path):
        """
        Load metric
        """
        try:
            logging.info("Loading metric ... ")
            metric = spio.loadmat(metric_path)['M']
            logging.info("full ")
        except:
            metric = np.loadtxt(metric_path)
            if metric.shape[0] % 1000:
                logging.warning("metric dimension: %d" % metric.shape[0])
            metric[metric < 1e-5] = 0
            logging.info("diagonal ")
            logging.info("metric loaded!")
        return metric

    @staticmethod
    def ordered(num_lst):
        """
        Check if a list is ordered
        """
        arr_lst = np.array(num_lst)
        return (arr_lst[1:] - arr_lst[:-1]).min() > 0

    def set_metric(self, metric):
        """
        Set metric
        """

        # non-metric
        if None == metric:
            self.distfunc = distFunc[self.distype]
            self.metric = None
        # diagnaol metric
        elif metric.ndim == 1:
            self.distfunc = DotProduct_DML_Diagonal
            self.dim_mask = (metric != 0)
            self.metric = metric[self.dim_mask]
        # dense metric
        elif metric.ndim == 2:
            self.distfunc = DotProduct_DML
            self.metric = metric
        # error
        else:
            raise Exception("BAD METRIC!")

    def add(self, featpath, featlst=None):
        """ Add features to index
        """
        raise Exception("%s.add(%s, %s) is not ready to use!\n" %
                        (self.__class__.__name__,
                         featpath.__name__, featlst.__name__))

    def query(self, featpath, qrylst=None, topk=None):
        """ Query
        """
        raise Exception("%s.query(%s, %s, %s) is not ready to use!\n" %
                        (self.__class__.__name__,
                         featpath.__name__, qrylst.__name__, topk.__name__))

    def adjust(self):
        """
        Adjust the indexer for better performance.
        Can be called at any place.
        """
        raise Exception("%s.adjust() is not ready to use!\n" %
                        (self.__class__.__name__))


def proc_nsmallest(v_par):
    """
    Select n smallest items
    """
    qid = v_par[0]
    dist = v_par[1]
    topk = v_par[2]
    return (qid, nsmallest(topk, dist))


def query_single2(v_par):
    """
    Single query2
    """
    qid = v_par[0]
    query = v_par[1]
    topk = v_par[2]
    indexer = v_par[3]

    featmat = indexer.featmat
    idxs = indexer.idxs
    metric = indexer.metric
    distfunc = indexer.distfunc

    tic(1)
    bmap = BitMap(featmat.shape[0])
    for wid in query.nonzero()[1]:
        for iid in idxs[wid]:
            bmap.set(iid)
    candlst = [iid for iid in range(bmap.size()) if bmap.test(iid)]
    logging.debug("\tTime for gathering list:    %.4fs" % toc(1))

    retk = min(featmat.shape[0], topk)
    ranked = np.ones((1, retk)) * -1
    # return empty results
    if len(candlst) == 0:
        return (qid, ranked)

    logging.debug("number of candidates: %d" % len(candlst))
    tic(2)
    if metric is None:
        distlst = distfunc(featmat[candlst], query)
    else:
        distlst = distfunc(featmat[candlst], metric, query)
    logging.debug("\tTime for gathering dist:    %.4fs" % toc(2))
    distlst = distlst.toarray().reshape(-1)
    logging.debug("\tTime for gathering dist:    %.4fs" % toc(2))

    tic(2)
    dist = np.array(distlst)
    cand = np.array(candlst)
    candtopk = min(dist.shape[0], retk)
    ranked[0, :candtopk] = cand[proc_nsmallest((0, dist, candtopk))[1]]
    if retk > candtopk:
        rand_res = set(range(featmat.shape[0])).difference(
            set(ranked[0, :candtopk]))
        ranked[0, candtopk:retk] = list(rand_res)[:retk-candtopk]

    logging.debug("\tTime for ranking:           %.4fs" % toc(2))

    logging.info("Time for querying:          %.4fs" % toc(1))

    return (qid, ranked)


def query_single3(v_par):
    """
    Single query3
    """
    qid = v_par[0]
    query = v_par[1]
    topk = v_par[2]
    indexer = v_par[3]

    idxs = indexer.idxs
    # metric = indexer.metric
    # distfunc = indexer.distfunc

    candidates = {}
    for (wid, wval) in izip(query.nonzero()[1], query.data):
        for (iid, val) in izip(idxs[wid][0], idxs[wid][1]):
            candidates[iid] = candidates.get(iid, 0) + \
                indexer.dist_core(wval, val)

    retk = min(indexer.nitems, topk)
    candtopk = min(len(candidates), retk)

    ranked = np.ones((1, retk)) * -1
    candidates = sorted(candidates.iteritems(), key=operator.itemgetter(1))
    ranked[0, :candtopk] = [item[0] for item in candidates]

    return (qid, np.array(ranked[:retk]).reshape(1, -1))


class Linear(Indexer):
    """ Linear scan
    """

    def __init__(self, distype='cosine', metric=None,
                 nproc=multiprocessing.cpu_count()):
        Indexer.__init__(self, distype, metric)
        self.featmat = None
        self.b_sparse = None
        self.pool = multiprocessing.Pool(processes=min(40, nproc))

    def __del__(self):
        """
        """
        self.pool.close()
        self.pool.join()
        Indexer.__del__(self)

    def add(self, featpath, featlst=None):
        """ Add features to index
        """
        if featlst is not None and not Indexer.ordered(featlst):
            logging.error("`featlst` is not ordered!")

        startnum = self.nitems

        loader = FeatLoader(featpath, featlst)
        logging.info("Loading first batch - ^^^^^")
        feat = loader.load_next()[0]
        self.b_sparse = 'todense' in dir(feat)
        logging.info("Loading first batch - $$$$$")

        logging.info("Initializing `featmat` - ^^^^^")
        if self.b_sparse:
            v_data = []
            v_i = []
            v_j = []
            logging.info("Initializing `featmat` - $$$$$ (sparse)")
        else:
            featmat = np.ndarray((len(featlst), feat.shape[1]), feat.dtype)
            logging.info("Initializing `featmat` - $$$$$ (dense)")

        logging.info("Loading batches - ^^^^^")
        nextpos = 0
        while feat is not None:
            if self.b_sparse:
                if self.dim_mask is not None:
                    logging.warning("Detect sparse feature with metric, ")
                    logging.warning("Reducing dimension: %d -> %d - ^^^^^" %
                                    (feat.shape[1], sum(self.dim_mask)))
                    feat = feat[:, self.dim_mask]
                    logging.warning("Reducing dimension: %d -> %d - $$$$$" %
                                    (feat.shape[1], sum(self.dim_mask)))
                v_data.append(feat.data)
                v_i.append(feat.nonzero()[0] + nextpos)
                v_j.append(feat.nonzero()[1])
            else:
                featmat[nextpos:nextpos + feat.shape[0]] = feat
            nextpos += feat.shape[0]
            feat = loader.load_next()[0]
        logging.info("Loading batches - $$$$$")

        self.nitems += nextpos

        if self.b_sparse:
            featmat = csr_matrix((np.hstack(v_data),
                                  (np.hstack(v_i), np.hstack(v_j))))
            logging.debug("sparse number: %d" % featmat.shape[0])

        if None == self.featmat:
            self.featmat = featmat
        else:
            self.featmat = np.vstack((self.featmat, featmat)) if not self.b_sparse \
                else sp.sparse.vstack((self.featmat, featmat), "csr")

        return (startnum, self.nitems)

    def get_dim(self):
        """ Get dimension of feature
        """
        return self.featmat.shape[1]

    def query_batch(self, query, topk):
        """ Query by batch
        """

        tic()
        if None == self.metric:
            dist = self.distfunc(self.featmat, query)
        else:
            dist = self.distfunc(self.featmat, self.metric, query)
        logging.debug("%s, %s, %.4f" %
                      (str(query.shape), str(self.featmat.shape), toc()))
        logging.debug("METRIC: %s" % str(self.metric))
        logging.debug("Time for computing dists: %.4fs" % toc())

        if self.b_sparse:
            dist = dist.toarray()

        if topk <= dist.shape[1]:
            # return dist.argsort(1)[:, :topk]

            ################################################################
            # The code bellow supposed to be faster than the above one,    #
            # but actually the numpy implementation is faster!!!           #
            ################################################################
            # ranked = dist.argsort(1)[:, :topk]
            # for i in range(ranked.shape[0]):
            #     print dist[i, ranked[i, :5]]
            # return ranked
            nqrys = dist.shape[0]
            ranked = np.ndarray((nqrys, topk), np.int64)
            parlst = [(qid, dist[qid, :], topk) for qid in range(nqrys)]
            for rid, res in enumerate(self.pool.imap_unordered(proc_nsmallest,
                                                               parlst, 10)):
                logging.debug("\t\t%d/%d" % (rid, nqrys))
                ranked[res[0]] = res[1]
            return ranked
        else:
            ranked = dist.argsort(1)
            # for i in range(ranked.shape[0]):
            #     print dist[i, ranked[i, :5]]
            return ranked

    def query(self, featpath, qrylst=None, topk=None):
        """ Query
        """
        if qrylst is not None and not Indexer.ordered(qrylst):
            logging.warning("`qrylst` is not ordered!")

        topk = min(topk, self.nitems) if topk else self.featmat.shape[0]

        ranked = np.ndarray((len(qrylst), topk), np.int64)

        """
        Important notes!!!
            if the qrylst is not ordered and the queries are loaded from
            multiple files, we need to reorder the ranked list in terms
            of queries
        """
        # how many files are the queries load from
        filenum = 0
        ranklst = []

        loader = FeatLoader(featpath, qrylst)
        q_start = 0
        query, idxlst = loader.load_next()
        while query is not None:
            if query.shape[0] == 0:
                query, idxlst = loader.load_next()
                continue

            filenum += 1
            ranklst += idxlst

            print query.shape

            if self.dim_mask is not None:
                logging.warning("Detect sparse query feature with metric!")
                logging.warning("Reducing dimension: %d -> %d - ^^^^^" %
                                (query.shape[1], sum(self.dim_mask)))
                query = query[:, self.dim_mask]
                logging.warning("Reducing dimension: %d -> %d - $$$$$" %
                                (query.shape[1], sum(self.dim_mask)))

            q_end = q_start + query.shape[0]

            qrybatch = 1000
            for i in range(q_start, q_end, qrybatch):
                i_end = min(i + qrybatch, q_end)
                tic()
                ranked[i:i_end, :] = self.query_batch(
                    query[i - q_start:i_end - q_start, :], topk)
                logging.info("\t%d~%d / %d, Time: %.4f" %
                             (i, i_end - 1, q_end, toc()))
            q_start = q_end

            query, idxlst = loader.load_next()

        # re-ordering the ranked list in terms of queries
        if not Indexer.ordered(qrylst) and filenum > 0:
            qrylst = qrylst.tolist()
            tmp_ranked = ranked
            ranked = np.ndarray((len(qrylst), topk), np.int64)
            for i in range(len(qrylst)):
                ranked[qrylst.index(ranklst[i]), :] = tmp_ranked[i, :]

        return ranked

    def adjust(self):
        """
        Adjust the indexer for better performance.
        Can be called at any place.
        """
        logging.debug("No additional process required for Indexer(%s)." %
                      self.__class__.__name__)


class Inverted(Linear):
    """ Inverted Index
    """

    @staticmethod
    def dotproduct_core(x, y):
        return - x * y

    @staticmethod
    def euclidean_core(x, y):
        return (x - y) ** 2

    def __init__(self, distype='cosine', metric=None,
                 nproc=multiprocessing.cpu_count()):
        Linear.__init__(self, distype, metric, nproc)
        logging.warning("Only sparse BoW is supported by %s currently" %
                        self.__class__.__name__)

        self.IDX_ARR_TYPE = 'I'
        self.VAL_ARR_TYPE = 'f'

        self.IDX_ARR2NPY = {
            'I': {4: np.uint32, 8: np.uint64},
            'L': {4: np.uint32, 8: np.uint64}}
        self.VAL_ARR2NPY = {'f': np.single, 'd': np.double}

        self.IDX_NPY_TYPE = self.IDX_ARR2NPY[self.IDX_ARR_TYPE][
            array.array(self.IDX_ARR_TYPE).itemsize]
        self.VAL_NPY_TYPE = self.VAL_ARR2NPY[self.VAL_ARR_TYPE]

        self.idxs = []
        self.featdim = None
        self.adjust = self.adjust0
        self.query_single = query_single3

        if distype == "dotproduct":
            self.dist_core = Inverted.dotproduct_core
        elif distype == "euclidean":
            self.dist_core = Inverted.euclidean_core
        else:
            raise Exception("Unsupported distance type for inverted index: %s"
                            % distype)

    def __del__(self):
        Linear.__del__(self)

    def add(self, featpath, featlst=None, featdim=None):
        """ Add features to index
        """
        if featlst is not None and not Indexer.ordered(featlst):
            logging.error("`featlst` is not ordered!")

        startnum = self.nitems

        loader = FeatLoader(featpath, featlst)

        logging.info("Loading first batch - ^^^^^")

        feat = loader.load_next()[0]
        b_sparse = 'todense' in dir(feat)
        featdim = featdim if featdim else feat.shape[1]

        if self.b_sparse is None:
            self.b_sparse = b_sparse
        elif self.b_sparse != b_sparse:
            raise Exception("Sparsity of features is not consistent!")

        if self.featdim is None:
            self.featdim = featdim
        elif self.b_sparse:
            self.featdim = max(self.featdim, featdim)
        elif self.featdim != featdim:
            raise Exception("Dimension of dense features is not consistent!")

        logging.info("Loading first batch - $$$$$")

        logging.info("Initializing `featmat` - ^^^^^")
        if self.b_sparse:
            rest = self.featdim - len(self.idxs)
            if rest > 0:
                self.idxs += [[array.array(self.IDX_ARR_TYPE),
                               array.array(self.VAL_ARR_TYPE)]
                              for wid in xrange(rest)]

            logging.info("Initializing `featmat` - $$$$$ (sparse)")
        else:
            featmat = np.ndarray((len(featlst), feat.shape[1]), feat.dtype)
            logging.info("Initializing `featmat` - $$$$$ (dense)")

        logging.info("Loading batches - ^^^^^")
        nextpos = 0
        time_cost = 0
        while feat is not None:
            tic()
            if self.b_sparse:
                if self.dim_mask is not None:
                    logging.warning("Detect sparse feature with metric, ")
                    logging.warning("Reducing dimension: %d -> %d - ^^^^^" %
                                    (feat.shape[1], sum(self.dim_mask)))
                    feat = feat[:, self.dim_mask]
                    logging.warning("Reducing dimension: %d -> %d - $$$$$" %
                                    (feat.shape[1], sum(self.dim_mask)))

                rids, cids = feat.nonzero()
                for (rid, cid, val) in izip(rids, cids, feat.data):
                    self.idxs[cid][0].append(rid)
                    self.idxs[cid][1].append(val)
            else:
                featmat[nextpos:nextpos + feat.shape[0]] = feat
            time_cost += toc()
            nextpos += feat.shape[0]
            feat = loader.load_next()[0]
        logging.info("Loading batches - $$$$$")
        logging.info("Time for indexing: %.4fs" % time_cost)

        self.nitems += nextpos

        if not self.b_sparse:
            if self.featmat is None:
                self.featmat = featmat
            else:
                self.featmat = np.vstack((self.featmat, featmat))

        return (startnum, self.nitems)

    def adjust2(self):
        logging.info("Adjusting index - ^^^^^")
        tic()
        self.idxs = [set() for wid in range(self.featmat.shape[1])]
        rids, cids = self.featmat.nonzero()
        logging.debug("\tpreparing done: %.4fs" % toc())
        for (rid, cid) in izip(rids, cids):
            self.idxs[cid].add(rid)
        logging.info("Adjusting index - $$$$$ (Time costs: %.4fs)!" % toc())

    def adjust3(self):
        logging.info("Adjusting index - ^^^^^")
        tic()
        self.idxs = [dict() for wid in range(self.featmat.shape[1])]
        rids, cids = self.featmat.nonzero()
        logging.debug("\tpreparing done: %.4fs" % toc())
        for (rid, cid, val) in izip(rids, cids, self.featmat.data):
            self.idxs[cid][rid] = val
        logging.info("Adjusting index - $$$$$ (Time costs: %.4fs)!" % toc())

    def adjust0(self):
        pass

    def query_batch(self, query, topk):
        nqrys = query.shape[0]
        dbsize = self.nitems
        topk = min(dbsize, topk)
        ranked = np.ndarray((nqrys, topk), np.int64)

        logging.info("Preparing parlst - ^^^^^")
        # shared_data_base = np.ctypeslib.as_ctypes(self.featmat.data)
        # shared_nz_i_base = np.ctypeslib.as_ctypes(self.featmat.nonzero()[0])
        # shared_nz_j_base = np.ctypeslib.as_ctypes(self.featmat.nonzero()[1])
        parlst = [(qid, query[qid], topk, self) for qid in range(nqrys)]
        logging.info("Preparing parlst - $$$$$")

        logging.info("Querying - ^^^^^")
        # for rid, res in enumerate(self.pool.imap_unordered(self.query_single,
        #                                                    parlst)):
        #     pinfo("\r\t\t\t%d/%d" % (rid, nqrys))
        #     ranked[res[0], :] = res[1]

        for qid in range(nqrys):
            tic(1)
            ranked[qid, :] = self.query_single(parlst[qid])[1]
            logging.info("Time: %.4fs" % toc(1))
        logging.info("Querying - $$$$$")
        return ranked


INDEXER_TABLE = {
    'linear':   Linear,
    'inverted': Inverted,
    'default':  Indexer}



#!/usr/bin/env python
# coding: utf-8

#########################################################################
#########################################################################

"""
   File Name: utils.py
      Author: Wan Ji
      E-mail: wanji@live.com
  Created on: Mon Jan 13 12:50:05 2014 CST
 Description:
"""

import os
import sys
import pickle
import bottleneck
import time
import logging

import numpy as np

from scipy.sparse.csr import csr_matrix

# default timer id is 0
TIME_TIC = {0: 0}


def tic(timer_id=0):
    """
    tic
    """
    global TIME_TIC
    TIME_TIC[timer_id] = time.time()


def toc(timer_id=0):
    """
    toc
    """
    return time.time() - TIME_TIC[timer_id]


def perr(msg):
    """ Print error message.
    """

    sys.stderr.write("%s" % msg)
    sys.stderr.flush()


def pinfo(msg):
    """ Print information message.
    """

    sys.stdout.write("%s" % msg)
    sys.stdout.flush()


def runcmd(cmd):
    """ Run command.
    """

    logging.debug("%s" % cmd)
    os.system(cmd)


def loadlist(lstpath, dtype=str):
    """ load text list
    """
    with open(lstpath) as lstfile:
        # return [[dtype(item) for item in line.strip()] for line in lstfile];
        return [dtype(line.strip()) for line in lstfile]


def load_lst_of_lst(lstpath, dtype=str):
    """ load list of list from file
    """
    lstoflst = []
    with open(lstpath) as lstfile:
        for line in lstfile:
            lstoflst.append([dtype(item) for item in line.split()])
    return lstoflst


def save_lst_of_lst(lstpath, lstoflst):
    """ save list of list to file
    """
    with open(lstpath, "w") as lstfile:
        for lst in lstoflst:
            lstfile.write(" ".join([str(item) for item in lst]) + "\n")


def save_lst_of_lst_plus(lstpath, lstoflst, shift=1):
    """ save list of list to file with shift
    this is for MATLAB in which index starts from 1
    """
    with open(lstpath, "w") as lstfile:
        for lst in lstoflst:
            lstfile.write(" ".join([str(item+shift) for item in lst]) + "\n")


def loadsvm(svmpath):
    """ Load data in SVM format into sparse matrix.
    """
    with open(svmpath) as svmf:
        cases = [line.split() for line in svmf]

    rows = len(cases)
    labels = np.zeros(rows, dtype=int)
    rids = []
    cids = []
    vals = []
    for i in range(rows):
        labels[i] = int(cases[i][0])
        cases[i] = [case.split(":") for case in cases[i][1:]]
        rids += [i for case in cases[i]]
        cids += [int(case[0]) for case in cases[i]]
        vals += [float(case[1]) for case in cases[i]]

    data = np.array(vals)
    ij = np.array([rids, cids])
    return (labels, csr_matrix((data, ij)))


def loadnpy(npypath):
    """ Load (label, feat) in NPY format.
    """
    with open(npypath, 'rb') as npyf:
        label = np.load(npyf)
        feat = np.load(npyf)
    return (label, feat)


def savenpy(label, feat, npypath):
    """ Save (label, feat) in NPY format.
    """
    with open(npypath, 'wb') as npyf:
        np.save(npyf, label)
        np.save(npyf, feat)


def loadmat(matpath):
    """ Load mat from file (txt format)
    """
    with open(matpath) as matf:
        [nrows, ncols] = [int(x) for x in matf.readline().split()]
        mat = np.empty((nrows, ncols))
        for i in range(nrows):
            mat[i, :] = np.array([float(x) for x in matf.readline().split()])
    return mat


def savemat(mat, matpath):
    """ Load mat from file (txt format)
    """
    with open(matpath, "w") as matf:
        matf.write("%d\t%d\n" % (mat.shape[0], mat.shape[1]))
        for i in range(mat.shape[0]):
            matf.write("%s\n" %
                       " ".join([str(x)
                                 for x in mat[i, :].reshape(-1).tolist()[0]]))


def pklload(pklpath):
    """ Load object from pickle file
    """
    with open(pklpath, 'rb') as pklf:
        return pickle.load(pklf)


def pkldump(obj, pklpath):
    """ Dump object to pickle file
    """
    with open(pklpath, 'wb') as pklf:
        return pickle.dump(obj, pklf)


def nsmallest(num, iterable):
    """ Get indices of the `num` smallest items in `iterable`
    """
    idxs = bottleneck.argpartsort(iterable, num)[:num]
    return idxs[iterable[idxs].argsort()]


# def compute_ap(ids, label):
#     """ Compute AP
#     """
#     old_recall    = 0.0;
#     old_precision = 1.0;
#     ap = 0.0;
#
#     vr = [old_recall]
#     vp = [old_precision]
#
#     intersect_size = 0.0;
#     posnum = float((ids == label).sum())
#
#     for i in range(ids.shape[0]):
#         if ids[i] == label:
#             intersect_size += 1.0
#
#             recall    = intersect_size / posnum
#             precision = intersect_size / (i+1)
#             vr.append(recall)
#             vp.append(precision)
#
#             ap += 0.5 * (recall - old_recall) * (old_precision + precision);
#
#             old_recall = recall;
#             old_precision = precision;
#             if intersect_size == posnum:
#                 break
#
#     return (ap, np.array(vr), np.array(vp));
#
# def compute_topk(ids, label, K):
#     """ Compute top-k
#     """
#     hitnum = 0.0
#     posnum = float((ids == label).sum())
#
#     precision = np.zeros(K)
#     recall    = np.zeros(K)
#     for i in range(K):
#         if ids[i] == label:
#             hitnum += 1.0
#         precision[i] = hitnum / (i+1)
#         recall[i]    = hitnum / posnum
#
#     return (precision, recall);


"""
Calculating difference vector
"""


def eucdiff(veca, vecb):
    """ Calculating difference for Euclidean distance
    """
    diff = veca - vecb
    # return similarity
    return -diff.multiply(diff)


def dotdiff(veca, vecb):
    """ Calculating difference for DOT similarity
    """
    return veca.multiply(vecb)



#!/usr/bin/env python
# coding: utf-8

#########################################################################
#########################################################################

"""
   File Name: bow2sparse.py
      Author: Wan Ji
      E-mail: wanji@live.com
  Created on: Mon Mar 24 11:33:07 2014 CST
"""
DESCRIPTION = """
This program is used for converting the BoW features from SVM format to
sparse matrix format in numpy.
"""

import os
import sys
import argparse
import multiprocessing


import numpy as np
from scipy.sparse import lil_matrix


def perr(msg):
    """ Print error message.
    """

    sys.stderr.write("%s" % msg)
    sys.stderr.flush()


def pinfo(msg):
    """ Print information message.
    """

    sys.stdout.write("%s" % msg)
    sys.stdout.flush()


def runcmd(cmd):
    """ Run command.
    """

    perr("%s\n" % cmd)
    os.system(cmd)


def getargs():
    """ Parse program arguments.
    """

    parser = argparse.ArgumentParser(
        description=DESCRIPTION, formatter_class=argparse.RawTextHelpFormatter)
    parser.add_argument('svmpath', type=str,
                        help='input: features in SVM format')
    parser.add_argument('npydir', type=str,
                        help='output: features in NPY format')
    parser.add_argument('rows', type=int,
                        help='number of rows')
    parser.add_argument('cols', type=int,
                        help='number of cols')
    parser.add_argument('-t', '--dtype', type=str, nargs='?',
                        default='float32',
                        help='data type')
    parser.add_argument('-b', '--blksz', type=int, nargs='?', default=12800,
                        help='block size')
    parser.add_argument('-c', '--ncpu', type=int, nargs='?',
                        default=multiprocessing.cpu_count(),
                        help='pool size (number of CPUs)')

    return parser.parse_args()


def proc_convert(v_par):
    """ proc
    """
    rid = v_par[0]
    line = v_par[1]
    dtype = v_par[2]
    items = [item.split(":") for item in line.split()[1:]]
    return (rid,
            [int(item[0]) for item in items],
            [dtype(item[1]) for item in items])


def main(args):
    """ Main entry.
    """

    dtype = eval('np.%s' % args.dtype)
    runcmd("mkdir -p %s" % args.npydir)

    pool = multiprocessing.Pool(processes=args.ncpu)
    pinfo("Converting ...\n")
    with open(args.svmpath) as svmf:
        for i in range(0, args.rows, args.blksz):
            nrows = min(args.blksz, args.rows - i)
            parlst = [(j, svmf.readline(), dtype) for j in range(nrows)]

            pinfo("\tInitializing matrix ... ")
            feat = lil_matrix((nrows, args.cols), dtype=dtype)
            pinfo("Done!\n")

            for rid, res in enumerate(pool.imap_unordered(proc_convert,
                                                          parlst)):
                if (rid % 100) == 0:
                    pinfo("\r\t%d/%d" % (i+rid, args.rows))
                feat.rows[res[0]] = res[1]
                feat.data[res[0]] = res[2]
            pinfo("\r\t%d/%d" % (i+nrows, args.rows))

            pinfo("\tConverting to csr_matrix ... ")
            feat = feat.tocsr()
            pinfo("Done!\n")

            pinfo("\tDumping ... ")
            npypath = os.path.sep.join([args.npydir,
                                        "bow_%d" % (i / args.blksz)])
            np.save(npypath, (feat.data, feat.nonzero()))
            pinfo("Done!\n")

    pinfo("\r\t%d/%d\tDone!\n" % (args.rows, args.rows))


if __name__ == '__main__':
    main(getargs())



#!/usr/bin/env python
# coding: utf-8

#########################################################################
#########################################################################

"""
   File Name: evaluation.py
      Author: Wan Ji
      E-mail: wanji@live.com
  Created on: Fri Mar 14 17:38:03 2014 CST
"""
DESCRIPTION = """
"""

import os
import logging
import multiprocessing
import numpy as np

import dataset
from utils import load_lst_of_lst, pinfo


class Evaluator(object):
    """ Evaluator
    """
    def __init__(self, nproc=min(80, multiprocessing.cpu_count())):
        """
        Evaluator
        """
        logging.info("\t++++++++++++++++++++++++++++++++")
        logging.info("\t+ Multiprocessing information  +")
        logging.info("\t+   Pool size: %3d             +" % (nproc))
        logging.info("\t++++++++++++++++++++++++++++++++")

        self.nproc = nproc
        self.pool = multiprocessing.Pool(processes=nproc)

    def __del__(self):
        self.pool.close()
        self.pool.join()

    def eval_rank(self, parlst, procfunc):
        """

        """
        nqrys = len(parlst)
        chunksize = min(max(1, nqrys / self.nproc), 100)

        eval_res = [[] for qid in range(nqrys)]
        logging.info("\t\tEvaluating %s - ^^^^^" % str(procfunc))
        for rid, res in enumerate(self.pool.imap_unordered(procfunc, parlst,
                                                           chunksize)):
            logging.debug("\t\t\t%d/%d" % (rid, nqrys))
            eval_res[res[0]] = res[1]
        logging.info("\t\tEvaluating %s - $$$$$" % str(procfunc))
        return np.array(eval_res)

    def eval_rank_sim2query(self, ranked, simlst, procfunc, params):
        """
        Evaluate ranked results based on similarity to query
        """
        nqrys = ranked.shape[0]
        parlst = zip(range(nqrys), simlst, list(ranked),
                     [params for qid in range(nqrys)])
        return self.eval_rank(parlst, procfunc)

    def eval_rank_posamb(self, ranked, poslst, amblst, procfunc, params):
        """
        Evaluate ranked results based on pos/amb list
        """
        nqrys = ranked.shape[0]
        parlst = zip(range(nqrys), poslst, amblst, list(ranked),
                     [params for qid in range(nqrys)])
        return self.eval_rank(parlst, procfunc)

    def eval_rank_ndcgk(self, ranked, simlst, params):
        """
        Evaluate map
        """
        return self.eval_rank_sim2query(ranked, simlst,
                                        compute_ndcg, params)

    def eval_rank_map(self, ranked, poslst, amblst, params):
        """
        Evaluate map
        """
        return self.eval_rank_posamb(ranked, poslst, amblst,
                                     compute_ap, params)

    def eval_rank_recallk(self, ranked, poslst, amblst, params):
        """
        Evaluate recall@k
        """
        return self.eval_rank_posamb(ranked, poslst, amblst,
                                     compute_recallk, params)

    def eval_rank_preck(self, ranked, poslst, amblst, params):
        """
        Evaluate prec@k
        """
        return self.eval_rank_posamb(ranked, poslst, amblst,
                                     compute_preck, params)


def compute_ap(v_list):
    """ Compute AP for a single query
    """
    qid = v_list[0]
    poslst = set(v_list[1])
    amblst = set(v_list[2])
    ranklst = v_list[3]
    # pardic = v_list[4]

    avgp = 0.0

    old_recall = 0.0
    old_precision = 1.0

    intersect_size = 0
    posnum = float(len(poslst))
    count = 0.0

    for resid in ranklst:
        if (resid in amblst):
            continue

        count += 1.0

        if (resid in poslst):
            intersect_size += 1

        if (len(poslst) == 0):
            recall = 0
            precision = 1.0
        else:
            recall = intersect_size / posnum
            precision = intersect_size / count

        avgp += (recall - old_recall) *    \
            ((old_precision + precision) / 2.0)

        if recall >= 1.0:
            break

        old_recall = recall
        old_precision = precision
    return (qid, avgp)


def compute_preck(v_list):
    """ Compute precision at K for a single query
    """

    qid = v_list[0]
    poslst = v_list[1]
    # amblst = v_list[2]
    ranklst = v_list[3]
    pardic = v_list[4]

    v_k = pardic['Ks']
    n_k = len(v_k)

    preck = np.ndarray(n_k)
    for i in range(n_k):
        cur_k = v_k[i]
        topk = set(ranklst[:cur_k])
        posnum = float(len(topk.intersection(poslst)))
        # ambnum = len(topk.intersection(amblst))
        # preck[i] = float(posnum) / (cur_k - ambnum)
        preck[i] = posnum / cur_k

    return (qid, preck)


def compute_recallk(v_list):
    """ Compute precision at K for a single query
    """

    qid = v_list[0]
    poslst = v_list[1]
    # amblst = v_list[2]
    ranklst = v_list[3]
    pardic = v_list[4]

    v_k = pardic['Ks']
    n_k = len(v_k)
    n_pos = len(poslst)

    recallk = np.ndarray(n_k)
    for i in range(n_k):
        cur_k = v_k[i]
        topk = set(ranklst[:cur_k])
        posnum = float(len(topk.intersection(poslst)))
        recallk[i] = posnum / n_pos

    return (qid, recallk)


def compute_ndcg(v_list):
    """ Compute precision at K for a single query
    """
    qid = v_list[0]
    s2qlst = v_list[1]
    ranklst = v_list[2]
    v_k = v_list[3]['Ks']

    ndcg_k = np.ndarray(len(v_k))

    max_k = v_k[-1]
    ranklst = ranklst[:max_k]
    rellst = np.zeros(max_k)
    for i in range(max_k):
        for val, lst in s2qlst.iteritems():
            if ranklst[i] in lst:
                rellst[i] = val
                break

    bestlst = []
    for val in sorted(s2qlst.keys(), reverse=True):
        lst = s2qlst[val]
        rest = max_k - len(bestlst)
        if rest <= len(lst):
            bestlst += [val for i in range(rest)]
            break
        bestlst += [val for i in range(len(lst))]
    if len(bestlst) < max_k:
        bestlst += [0 for i in range(max_k - len(bestlst))]
    bestlst = np.array(bestlst)
    # print np.vstack((bestlst, rellst))

    old_k = 0
    old_z = 0
    old_g = 0

    for i in range(len(v_k)):
        cur_k = v_k[i]
        cur_range = np.array(range(old_k, cur_k))
        cur_g = old_g + ((2 ** rellst[cur_range] - 1) /
                         np.log2(cur_range + 2)).sum()
        cur_z = old_z + ((2 ** bestlst[cur_range] - 1) /
                         np.log2(cur_range + 2)).sum()

        ndcg_k[i] = cur_g / cur_z

        old_k = cur_k
        old_z = cur_z
        old_g = cur_g

    return (qid, ndcg_k)


EVALUATORS = {
    'ndcgk':    compute_ndcg,
    'preck':   compute_preck,
    'recallk': compute_recallk,
    'map':     compute_ap}


def res_print(results):
    """
    Printing results
    """
    pinfo("\n\n")
    pinfo("================ Results ================\n")
    for trg, res in results.iteritems():
        pinfo("%s:\n" % trg)
        # print " ".join("%.4f" % x for x in list(np.array(res).mean(0)))
        try:
            pinfo("%s\n" % " ".join("%.4f" % x for x in np.array(res).mean(0)))
        except TypeError as err:
            logging.debug(err.message)
            pinfo("%f\n" % np.array(res).mean(0))
        # print np.array(res)
        pinfo("-----------------------------------------\n")


def eval_rank(ranked, datadir, targets,
              nproc=multiprocessing.cpu_count()):
    """ Evaluate mAP
    """

    eval_posamb = ['map', 'preck', 'recallk']
    eval_s2q = ['ndcgk']
    eval_posamb_hit = []
    eval_s2q_hit = []
    for trg in targets:
        if trg in eval_posamb:
            eval_posamb_hit.append(trg)
        if trg in eval_s2q:
            eval_s2q_hit.append(trg)

    # number of queries
    nqrys = ranked.shape[0]
    # # convert ranked results to list
    # ranklst = list(ranked)
    # evaluation results
    eval_res = {}

    logging.info("\t++++++++++++++++++++++++++++++++++++++++++++++++++++++++")
    logging.info("\t+ Multiprocessing information                          +")
    pool_size = min(80, nproc)
    chunksize = min(max(1, nqrys / pool_size), 100)
    logging.info("\t+  Pool size: %3d      Chunk size: %3d                 +" %
                 (pool_size, chunksize))
    logging.info("\t++++++++++++++++++++++++++++++++++++++++++++++++++++++++")

    pool = multiprocessing.Pool(processes=pool_size)
    if len(eval_s2q_hit) > 0:
        logging.info("\t################################################")
        logging.info("\t# Evaluating by similarity to queries ##########")
        logging.info("\t################################################")

        # loading similarity to query
        (s2qdir, s2qpre) = os.path.split(dataset.Builder.S2Q_LIST)
        s2qdir = os.path.sep.join([datadir, s2qdir])
        s2qpre += '.'

        simlst = [{} for qid in range(nqrys)]
        for fname in os.listdir(s2qdir):
            # print s2qpre, fname
            if (s2qpre in fname) and (fname[-2:] != "_1"):
                simval = float(fname.split(s2qpre)[-1])
                simpath = os.path.sep.join([s2qdir, fname])
                v_lst = load_lst_of_lst(simpath, int)
                for qid in range(nqrys):
                    simlst[qid][simval] = v_lst[qid]

        for trg in eval_s2q_hit:
            parlst = zip(range(nqrys), simlst, list(ranked),
                         [targets[trg] for qid in range(nqrys)])
            eval_res[trg] = [[] for qid in range(nqrys)]
            logging.info("\t\tEvaluating %s - ^^^^^" % trg)
            for rid, res in enumerate(pool.imap_unordered(EVALUATORS[trg],
                                                          parlst, chunksize)):
                logging.debug("\t\t\t%d/%d" % (rid, nqrys))
                eval_res[trg][res[0]] = res[1]
            # for par in parlst:
            #     res = EVALUATORS[trg](par)
            #     pinfo("\r\t\t\t%d/%d" % (par[0], nqrys))
            #     eval_res[trg][res[0]] = res[1]
            eval_res[trg] = np.array(eval_res[trg])
            logging.info("\t\tEvaluating %s - $$$$$" % trg)

    if len(eval_posamb_hit) > 0:
        logging.info("\t################################################")
        logging.info("\t# Evaluating by positive/ambiguous list ########")
        logging.info("\t################################################")

        pospath = os.path.sep.join([datadir, dataset.Builder.POS_LIST])
        ambpath = os.path.sep.join([datadir, dataset.Builder.AMB_LIST])
        poslst = load_lst_of_lst(pospath, int)
        amblst = load_lst_of_lst(ambpath, int)

        for trg in eval_posamb_hit:
            # if trg in ['preck', 'recallk'] and ranked.shape[0] == 55:
            #     parlst = zip(range(nqrys), poslst, amblst, list(ranked[:, 1:]),
            #                 [targets[trg] for qid in range(nqrys)])
            # else:
            #     parlst = zip(range(nqrys), poslst, amblst, list(ranked),
            #                 [targets[trg] for qid in range(nqrys)])
            parlst = zip(range(nqrys), poslst, amblst, list(ranked),
                         [targets[trg] for qid in range(nqrys)])
            eval_res[trg] = [[] for qid in range(nqrys)]
            logging.info("\t\tEvaluating %s - ^^^^^" % trg)
            for rid, res in enumerate(pool.imap_unordered(EVALUATORS[trg],
                                                          parlst, chunksize)):
                logging.debug("\t\t\t%d/%d" % (rid, nqrys))
                eval_res[trg][res[0]] = res[1]
            # for par in parlst:
            #     res = EVALUATORS[trg](par)
            #     pinfo("\r\t\t\t%d/%d" % (par[0], nqrys))
            #     eval_res[trg][res[0]] = res[1]
            eval_res[trg] = np.array(eval_res[trg])
            logging.info("\t\tEvaluating %s - $$$$$" % trg)

    pool.close()
    pool.join()
    return eval_res



#!/usr/bin/env python
# coding: utf-8

################################################################
# Dataset Builder ##############################################
################################################################

"""
   File Name: dataset.py
      Author: Wan Ji
      E-mail: wanji@live.com
  Created on: Sun Mar 16 12:03:09 2014 CST
"""
DESCRIPTION = """
"""

import random
from utils import *
from proto.dataset_pb2 import Sim2Query, Search, Dataset
from google.protobuf import text_format


class Builder(object):
    """ Dataset builder
    """

    # basic info
    IMAGE_LIST = 'image.lst'
    LABEL_LIST = 'label.lst'
    IMAGE_DATA = 'image.dat'
    IMAGE_DIR = 'image_dir'

    # search related
    DB_IMG_LIST = 'search/db.lst'
    QRY_IMG_LIST = 'search/query.lst'
    S2Q_LIST = 'search/sim2qry.lst'
    POS_LIST = 'search/pos.lst'
    AMB_LIST = 'search/amb.lst'
    TRAIN_LIST = 'search/train.lst'

    def __init__(self):
        # protobuf
        self.dataset = Dataset()
        self.search = Search()

        # path or name of images
        self.imglst = None
        # label of images
        self.lbslst = None
        # images in database
        self.dblst = None
        # images used as queries
        self.qrylst = None
        # similarity of database images to queries
        self.s2qlst = None
        # image similar to queries
        self.poslst = None
        # image ambiguous to queries
        self.amblst = None
        # images for training
        self.trainlst = None

        # print the doc string, which includes the PRE-REQURIES of
        # the source dir
        print self.__doc__

    def __def__(self):
        pass

    @classmethod
    def set_parser(cls, parser):
        """ Setup the command line parser
        """
        parser.add_argument('trg_dir', type=str,
                            help='target dir of dataset')

    @classmethod
    def get_name(cls, filepath):
        """ Get name of the file, excluding folder and extension
        """
        return ".".join(os.path.split(filepath)[1].split('.')[:-1])

    @staticmethod
    def ilst(arr):
        """
        Convert an array to int32 list
        """
        return [int(item) for item in arr]

    def generate_protobuf(self):
        """
        Packing and generate protobuf file
        """
        pinfo("Packing dataset protobuf ...\n")
        self.dataset.name = "Hi"
        self.dataset.comment = "Bye"
        self.dataset.images.extend(self.imglst)
        print self.dataset.labels
        print type(list(self.lbslst))
        print type(list(self.lbslst)[0])
        self.dataset.labels.extend(self.ilst(self.lbslst))
        pinfo("Done!\n")

        pinfo("Packing search protobuf ...\n")
        self.search.dblst.extend(self.ilst(self.dblst))
        self.search.trainlst.extend(self.ilst(self.trainlst))

        for qid in range(len(self.qrylst)):
            query = self.search.qrylst.add()
            query.query = self.qrylst[qid]
            query.poslst.extend(self.ilst(self.poslst[qid]))
            query.amblst.extend(self.ilst(self.amblst[qid]))

            for val, lsts in self.s2qlst.iteritems():
                sim2qrylst = query.sim2qrylst.add()
                sim2qrylst.sim2qry = val
                sim2qrylst.imglst.extend(self.ilst(lsts[qid]))
        pinfo("Done!\n")


    def generate(self, trgdir):
        """ Generate the dataset related files.
        """

        self.generate_protobuf()

        with open("%s/dataset.proto" % trgdir, "w") as dsf:
            dsf.write(text_format.MessageToString(self.dataset))

        with open("%s/search.proto" % trgdir, "w") as shf:
            shf.write(text_format.MessageToString(self.search))

        pinfo("Generating dataset files ")
        # create target dir
        os.system("mkdir -p %s/search" % trgdir)

        # store image list
        if None != self.imglst:
            np.savetxt(os.sep.join([trgdir, Builder.IMAGE_LIST]),
                       self.imglst, '%s')
            pinfo(".")
        if None != self.lbslst:
            np.savetxt(os.sep.join([trgdir, Builder.LABEL_LIST]),
                       self.lbslst, '%s')
            pinfo(".")
        if None != self.dblst:
            np.savetxt(os.sep.join([trgdir, Builder.DB_IMG_LIST]),
                       self.dblst, '%s')
            np.savetxt(os.sep.join([trgdir, Builder.DB_IMG_LIST+"_1"]),
                       [iid+1 for iid in self.dblst], '%s')
            pinfo(".")
        if None != self.qrylst:
            np.savetxt(os.sep.join([trgdir, Builder.QRY_IMG_LIST]),
                       self.qrylst, '%s')
            np.savetxt(os.sep.join([trgdir, Builder.QRY_IMG_LIST+"_1"]),
                       [iid+1 for iid in self.qrylst], '%s')
            pinfo(".")
        if None != self.trainlst:
            np.savetxt(os.sep.join([trgdir, Builder.TRAIN_LIST]),
                       self.trainlst, '%s')
            np.savetxt(os.sep.join([trgdir, Builder.TRAIN_LIST+"_1"]),
                       [iid+1 for iid in self.trainlst], '%s')
            pinfo(".")
        if None != self.s2qlst:
            for val, lsts in self.s2qlst.iteritems():
                save_lst_of_lst(os.sep.join([trgdir, Builder.S2Q_LIST +
                                             ".%.2f" % val]), lsts)
                save_lst_of_lst_plus(os.sep.join([trgdir, Builder.S2Q_LIST +
                                                  ".%.2f_1" % val]), lsts)
            pinfo(".")
        if None != self.poslst:
            save_lst_of_lst(os.sep.join([trgdir, Builder.POS_LIST]),
                            self.poslst)
            save_lst_of_lst_plus(os.sep.join([trgdir, Builder.POS_LIST+"_1"]),
                                 self.poslst)
            pinfo(".")
        if None != self.amblst:
            save_lst_of_lst(os.sep.join([trgdir, Builder.AMB_LIST]),
                            self.amblst)
            save_lst_of_lst_plus(os.sep.join([trgdir, Builder.AMB_LIST+"_1"]),
                                 self.amblst)
            pinfo(".")
        pinfo(" Done!\n")


class Oxford(Builder):
    """ Oxford-dataset builder.
    Download: http://www.robots.ox.ac.uk/~vgg/data/oxbuildings/

    !!NOTE!! `all_souls_000183.jpg` appeares in both `all_souls` and
             `radcliffe_camera` classes
    """

    ALL_LANDMARKS = ['all_souls', 'ashmolean', 'balliol', 'bodleian',
                     'christ_church', 'cornmarket', 'hertford', 'keble',
                     'magdalen', 'pitt_rivers', 'radcliffe_camera']

    def __init__(self, args):
        """ Parse the Oxford dataset
        """
        Builder.__init__(self)

        pinfo("Loading image list ... ")
        # convert the image path into relative path to the target dir
        imnames = [item.split(".jpg")[0] for item in
                   sorted(os.listdir(args.img_dir))]
        self.imglst = [os.path.relpath(os.sep.join([args.img_dir,
                                                    imname + ".jpg"]),
                                       args.trg_dir) for imname in imnames]
        self.lbslst = np.zeros(len(self.imglst), np.int32)
        pinfo("Done!\n")

        self.qrylst = []
        self.poslst = []
        self.amblst = []
        self.s2qlst = {args.good: [], args.ok: [], args.junk: []}

        clsnum = len(Oxford.ALL_LANDMARKS)
        # class id start from 1, 0 denotes background images
        for (clsid, landmark) in zip(range(1, clsnum + 1),
                                     Oxford.ALL_LANDMARKS):
            pinfo("\t%s\n" % landmark)
            for i in range(5):
                gtf = "%s_%d" % (landmark, i+1)

                qname = np.loadtxt("%s/%s_query.txt" % (args.gt_dir, gtf), dtype=np.str).\
                    tolist()[0].replace('oxc1_', '')
                qid = imnames.index(qname)
                self.qrylst.append(qid)

                good_names = loadlist("%s/%s_good.txt" % (args.gt_dir, gtf))
                good_ids = [imnames.index(x) for x in good_names]
                self.s2qlst[args.good].append(good_ids)

                ok_names = loadlist("%s/%s_ok.txt" % (args.gt_dir, gtf))
                ok_ids = [imnames.index(x) for x in ok_names]
                self.s2qlst[args.ok].append(ok_ids)

                cur_poslst = np.array(ok_ids + good_ids)
                # cur_poslst = cur_poslst[cur_poslst != qid]
                self.poslst.append(list(cur_poslst))

                junk_names = loadlist("%s/%s_junk.txt" % (args.gt_dir, gtf))
                junk_ids = [imnames.index(x) for x in junk_names]
                self.s2qlst[args.junk].append(junk_ids)
                self.amblst.append(junk_ids)
            self.lbslst[self.poslst[-1]] = clsid

        self.dblst = np.array(range(len(self.imglst)))

        self.trainlst = list(set(range(len(self.imglst))).difference(set(self.qrylst)))
        # self.trainlst = []
        # for classid in range(len(Oxford.ALL_LANDMARKS)):
        #     # select images with the same IDs, BUT NOT IN THE QUERY LIST
        #     candidates = [iid for iid in self.dblst[self.lbslst == (classid + 1)]
        #                   if iid not in self.qrylst]
        #     random.shuffle(candidates)
        #     self.trainlst += candidates[:args.selpos]

        # candidates = list(self.dblst[self.lbslst == 0])
        # random.shuffle(candidates)
        # self.trainlst += candidates[:args.selneg]

        pinfo("\n")

    def __del__(self):
        pass

    @classmethod
    def set_parser(cls, parser):
        Builder.set_parser(parser)
        parser.add_argument('img_dir', type=str,
                            help="""dir containing images extracted from
                            `oxbuild_images.tgz`""")
        parser.add_argument('gt_dir', type=str,
                            help="""dir containing groundtruth extracted from
                            `gt_files_170407.tgz`""")
        parser.add_argument('--good', type=float, default=3.0,
                            help="gain value for `good` results")
        parser.add_argument('--ok', type=float, default=2.0,
                            help="gain value for `ok` results")
        parser.add_argument('--junk', type=float, default=1.0,
                            help="gain value for `junk` results")
        parser.add_argument('--selpos', type=int, default=7,
                            help="NO. of positive image for each class")
        parser.add_argument('--selneg', type=int, default=500,
                            help="number of selected negative")


class OxfordM(Oxford):
    """ Metric Learning Edition of `Oxford`
    """

    # selected classes for metric learning
    SEL_LANDMARKS = ['all_souls', 'ashmolean', 'bodleian', 'christ_church',
                     'hertford', 'magdalen', 'radcliffe_camera']
    # SEL_LANDMARKS = Oxford.ALL_LANDMARKS
    #

    def __init__(self, args):
        Oxford.__init__(self, args)

        all_classes = ['background'] + Oxford.ALL_LANDMARKS

        pinfo("Preparing for metric learning ... ")
        self.trainlst = []
        qry_lbs = self.lbslst[self.qrylst]
        qry_mask = np.zeros(len(self.qrylst))
        for sel_landmark in OxfordM.SEL_LANDMARKS:
            classid = all_classes.index(sel_landmark)
            qry_mask[qry_lbs == classid] = 1
            # select images with the same IDs, BUT NOT IN THE QUERY LIST
            candidates = [iid for iid in self.dblst[self.lbslst == (classid+1)]
                          if iid not in self.qrylst]
            random.shuffle(candidates)
            self.trainlst += candidates[:args.selpos]

        candidates = list(self.dblst[self.lbslst == 0])
        random.shuffle(candidates)
        self.trainlst += candidates[:500]

        self.qrylst = [self.qrylst[qid] for qid in range(len(self.qrylst))
                       if qry_mask[qid] > 0]
        self.poslst = [self.poslst[qid] for qid in range(len(self.poslst))
                       if qry_mask[qid] > 0]
        self.amblst = [self.amblst[qid] for qid in range(len(self.amblst))
                       if qry_mask[qid] > 0]

        self.dblst = [iid for iid in self.dblst
                      if iid not in self.trainlst]
        for i in range(len(self.qrylst)):
            self.poslst[i] = [iid for iid in self.poslst[i]
                              if iid not in self.trainlst]
            self.amblst[i] = [iid for iid in self.amblst[i]
                              if iid not in self.trainlst]
        for key in self.s2qlst.keys():
            self.s2qlst[key] = [self.s2qlst[key][qid] for qid
                                in range(len(self.s2qlst[key]))
                                if qry_mask[qid] > 0]
            for i in range(len(self.qrylst)):
                self.s2qlst[key][i] = [iid for iid in self.s2qlst[key][i]
                                       if iid not in self.trainlst]
        self.dblst = [iid for iid in self.dblst
                      if iid not in self.trainlst]
        pinfo("Done!\n")

    @classmethod
    def set_parser(cls, parser):
        Oxford.set_parser(parser)


class Paris(Builder):
    """ Paris-dataset builder.
    Download: http://www.robots.ox.ac.uk/~vgg/data/parisbuildings/
    """

    ALL_LANDMARKS = ['defense', 'eiffel', 'invalides', 'louvre', 'moulinrouge',
                     'museedorsay', 'notredame', 'pantheon', 'pompidou',
                     'sacrecoeur', 'triomphe']

    def __init__(self, args):
        """ Parse the Parse dataset
        """
        Builder.__init__(self)

        pinfo("Loading image list ... ")
        self.imglst = []
        # convert the image path into relative path to the target dir
        for dpath, _, files in os.walk(args.img_dir):
            for fpath in files:
                self.imglst.append(os.path.relpath(os.path.join(dpath, fpath),
                                                   args.trg_dir))
        self.imglst = sorted(self.imglst)
        imnames = [Builder.get_name(path) for path in self.imglst]
        self.lbslst = np.zeros(len(self.imglst), np.int32)
        pinfo("Done!\n")

        self.qrylst = []
        self.poslst = []
        self.amblst = []
        self.s2qlst = {args.good: [], args.ok: [], args.junk: []}

        clsnum = len(Oxford.ALL_LANDMARKS)
        # class id start from 1, 0 denotes background images
        for (clsid, landmark) in zip(range(1, clsnum + 1),
                                     Paris.ALL_LANDMARKS):
            pinfo("\t%s\n" % landmark)
            for i in range(5):
                gtf = "%s_%d" % (landmark, i+1)

                qname = np.loadtxt("%s/%s_query.txt" % (args.gt_dir, gtf), dtype=np.str).\
                    tolist()[0].replace('oxc1_', '')
                qid = imnames.index(qname)
                self.qrylst.append(qid)

                good_names = loadlist("%s/%s_good.txt" % (args.gt_dir, gtf))
                good_ids = [imnames.index(x) for x in good_names]
                self.s2qlst[args.good].append(good_ids)

                ok_names = loadlist("%s/%s_ok.txt" % (args.gt_dir, gtf))
                ok_ids = [imnames.index(x) for x in ok_names]
                self.s2qlst[args.ok].append(ok_ids)

                cur_poslst = np.array(ok_ids + good_ids)
                # cur_poslst = cur_poslst[cur_poslst != qid]
                self.poslst.append(list(cur_poslst))

                junk_names = loadlist("%s/%s_junk.txt" % (args.gt_dir, gtf))
                junk_ids = [imnames.index(x) for x in junk_names]
                self.s2qlst[args.junk].append(junk_ids)
                self.amblst.append(junk_ids)
            self.lbslst[self.poslst[-1]] = clsid
            print clsid, len(self.poslst[-1])

        self.dblst = np.array(range(len(self.imglst)))

        self.trainlst = list(set(range(len(self.imglst))).difference(set(self.qrylst)))
        # self.trainlst = []
        # for classid in range(len(Paris.ALL_LANDMARKS)):
        #     # select images with the same IDs, BUT NOT IN THE QUERY LIST
        #     candidates = [iid for iid in self.dblst[self.lbslst == (classid+1)]
        #                   if iid not in self.qrylst]
        #     random.shuffle(candidates)
        #     self.trainlst += candidates[:args.selpos]

        # candidates = list(self.dblst[self.lbslst == 0])
        # random.shuffle(candidates)
        # self.trainlst += candidates[:args.selneg]

        pinfo("\n")

    def __del__(self):
        pass

    @classmethod
    def set_parser(cls, parser):
        Builder.set_parser(parser)
        parser.add_argument('img_dir', type=str,
                            help="""dir containing images extracted from
                            `paris_1.tgzk` and `paris_2.tgz`""")
        parser.add_argument('gt_dir', type=str,
                            help="""dir containing groundtruth extracted from
                            `paris_120310.tgz`""")
        parser.add_argument('--good', type=float, default=3.0,
                            help="gain value for `good` results")
        parser.add_argument('--ok', type=float, default=2.0,
                            help="gain value for `ok` results")
        parser.add_argument('--junk', type=float, default=1.0,
                            help="gain value for `junk` results")
        parser.add_argument('--selpos', type=int, default=15,
                            help="NO. of positive image for each class")
        parser.add_argument('--selneg', type=int, default=500,
                            help="number of selected negative")


class ParisM(Paris):
    """ Metric Learning Edition of `Paris`
    """

    # selected classes for metric learning
    SEL_LANDMARKS = Paris.ALL_LANDMARKS

    def __init__(self, args):
        Paris.__init__(self, args)

        all_classes = ['background'] + Paris.ALL_LANDMARKS

        pinfo("Preparing for metric learning ... ")
        self.trainlst = []
        qry_lbs = self.lbslst[self.qrylst]
        qry_mask = np.zeros(len(self.qrylst))
        for sel_landmark in ParisM.SEL_LANDMARKS:
            classid = all_classes.index(sel_landmark)
            qry_mask[qry_lbs == classid] = 1
            candidates = [iid for iid in self.dblst[self.lbslst == (classid+1)]
                          if iid not in self.qrylst]
            random.shuffle(candidates)
            self.trainlst += candidates[:args.selpos]

        candidates = list(self.dblst[self.lbslst == 0])
        random.shuffle(candidates)
        self.trainlst += candidates[:500]

        self.qrylst = [self.qrylst[qid] for qid in range(len(self.qrylst))
                       if qry_mask[qid] > 0]
        self.poslst = [self.poslst[qid] for qid in range(len(self.poslst))
                       if qry_mask[qid] > 0]
        self.amblst = [self.amblst[qid] for qid in range(len(self.amblst))
                       if qry_mask[qid] > 0]

        self.dblst = [iid for iid in self.dblst
                      if iid not in self.trainlst]
        for i in range(len(self.qrylst)):
            self.poslst[i] = [iid for iid in self.poslst[i]
                              if iid not in self.trainlst]
            self.amblst[i] = [iid for iid in self.amblst[i]
                              if iid not in self.trainlst]
        for key in self.s2qlst.keys():
            self.s2qlst[key] = [self.s2qlst[key][qid] for qid
                                in range(len(self.s2qlst[key]))
                                if qry_mask[qid] > 0]
            for i in range(len(self.qrylst)):
                self.s2qlst[key][i] = [iid for iid in self.s2qlst[key][i]
                                       if iid not in self.trainlst]
        self.dblst = [iid for iid in self.dblst
                      if iid not in self.trainlst]
        pinfo("Done\n")

    @classmethod
    def set_parser(cls, parser):
        Paris.set_parser(parser)


class Holidays(Builder):
    """ Holidays-dataset builder.
    Download: http://lear.inrialpes.fr/~jegou/data.php
    """

    def __init__(self, args):
        """ Parse the Parse dataset
        """
        Builder.__init__(self)

        pinfo("Loading image list ... ")
        # convert the image path into relative path to the target dir
        imnames = [item.split(".jpg")[0] for item in
                   sorted(os.listdir(args.img_dir))]
        self.imglst = [os.path.relpath(os.sep.join([args.img_dir,
                                                    imname + ".jpg"]),
                                       args.trg_dir) for imname in imnames]
        pinfo("Done!\n")

        self.qrylst = []
        self.poslst = []
        self.amblst = []
        self.s2qlst = {args.good: [], args.ok: [], args.junk: []}

        for imname in imnames:
            imidx = imnames.index(imname)
            print imidx, len(self.qrylst)
            if (int(imname) % 100 == 0):
                self.qrylst.append(imidx)
                self.poslst.append([])
                self.amblst.append([imidx])
                self.s2qlst[args.good].append([])
                self.s2qlst[args.ok].append([])
                self.s2qlst[args.junk].append([])
            else:
                self.poslst[len(self.qrylst)-1].append(imidx)
                self.s2qlst[args.ok][len(self.qrylst)-1].append(imidx)

        self.dblst = range(len(self.imglst))
        pinfo("\n")

    def __del__(self):
        pass

    @classmethod
    def set_parser(cls, parser):
        Builder.set_parser(parser)
        parser.add_argument('img_dir', type=str,
                            help="""dir containing images extracted from
                            `jpg1.tar.gz` and `jpg2.tar.gz`""")
        parser.add_argument('--good', type=float, default=3.0,
                            help="gain value for `good` results")
        parser.add_argument('--ok', type=float, default=2.0,
                            help="gain value for `ok` results")
        parser.add_argument('--junk', type=float, default=1.0,
                            help="gain value for `junk` results")


class UKBench(Builder):
    """ UKBench-dataset builder.
    Download: http://vis.uky.edu/~stewe/ukbench/
    """

    def __init__(self, args):
        """ Parse the Parse dataset
        """
        Builder.__init__(self)

        pinfo("Loading image list ... ")
        # convert the image path into relative path to the target dir
        imnames = [item.split(".jpg")[0] for item in
                   sorted(os.listdir(args.img_dir))]
        self.imglst = [os.path.relpath(os.sep.join([args.img_dir,
                                                    imname + ".jpg"]),
                                       args.trg_dir) for imname in imnames]
        pinfo("Done!\n")

        self.qrylst = []
        self.poslst = []
        self.amblst = []

        for idx in range(0, len(imnames), 4):
            self.qrylst.append(idx)
            self.poslst.append(range(idx, idx+4))
            self.amblst.append([])

        self.dblst = range(len(self.imglst))
        pinfo("\n")

    def __del__(self):
        pass

    @classmethod
    def set_parser(cls, parser):
        Builder.set_parser(parser)
        parser.add_argument('img_dir', type=str,
                            help="""dir containing full sized images extracted from
                            `ukbench.zip`""")


class Caltech(Builder):
    """ Caltech101/256-dataset builder.
    Download: http://www.vision.caltech.edu/Image_Datasets/Caltech101/
              http://www.vision.caltech.edu/Image_Datasets/Caltech256/

    NOTE: the image list are generated by another program.
        The list contains lines of image information:
            `path_of_image_0` `class_label_of_image_0`
            `path_of_image_1` `class_label_of_image_1`
            `path_of_image_2` `class_label_of_image_2`
            ...
    """

    def __init__(self, args, pos=1):
        """
        """
        Builder.__init__(self)

        self.s2qlst = {pos: []}

        pinfo("Loading image list ")
        pinfo(".")
        imglst = [line.split() for line in loadlist(args.imglst)]
        pinfo(".")
        self.imglst = [line[0] for line in imglst]
        pinfo(".")
        raw_lbs = [int(line[1]) for line in imglst]
        uniqlbs = list(np.unique(raw_lbs))
        self.lbslst = np.array([uniqlbs.index(lbl) for lbl in raw_lbs])
        pinfo(".")
        pinfo(" Done!\n")

        self.trainlst = np.ndarray(0, np.int64)
        self.dblst = np.ndarray(0, np.int64)
        self.qrylst = np.ndarray(0, np.int64)

        imgnum = len(self.imglst)
        clsnum = len(uniqlbs)
        imgrange = np.array(range(imgnum))
        for i in range(clsnum):
            cur_ids = imgrange[self.lbslst == i]
            np.random.shuffle(cur_ids)
            self.trainlst = np.hstack((self.trainlst, cur_ids[:args.trnum]))
            self.qrylst = np.hstack((self.qrylst,
                                     cur_ids[args.trnum:
                                             args.trnum + args.tsnum]))
            self.dblst = np.hstack((self.dblst,
                                    cur_ids[args.trnum:
                                            args.trnum + args.tsnum]))

        pinfo("\tGenerating label list ")
        pinfo(".")
        self.dlblst = self.lbslst[self.dblst]
        pinfo(".")
        self.qlblst = self.lbslst[self.qrylst]
        pinfo(".")
        pinfo(" Done!\n")

        pinfo("\tGenerating pos list \n")
        self.poslst = []
        self.amblst = []
        qrynum = len(self.qrylst)
        for qid in range(qrynum):
            if qid % 10 == 0:
                pinfo("\r\t\t%d/%d" % (qid, qrynum))
            self.poslst.append(self.dblst[self.dlblst == self.qlblst[qid]])
            self.amblst.append([])
        self.s2qlst[pos] = self.poslst
        pinfo("\r\t\t%d/%d\tDone!\n" % (len(self.qrylst), len(self.qrylst)))

    def __del__(self):
        pass

    @classmethod
    def set_parser(cls, parser):
        Builder.set_parser(parser)
        parser.add_argument('imglst', type=str,
                            help="image image list")
        parser.add_argument('trnum', type=int, nargs='?', default=30,
                            help="NO. of training images per class")
        parser.add_argument('tsnum', type=int, nargs='?', default=20,
                            help="NO. of test images per class")


class ImageNet2012(Builder):
    """ ImageNet 2012 dataset builder.
    Download: http://www.image-net.org/challenges/LSVRC/2012/

    NOTE: the training/validation image list are generated by another program.
        both list contain lines of image information:
            `path_of_image_0` `class_label_of_image_0`
            `path_of_image_1` `class_label_of_image_1`
            `path_of_image_2` `class_label_of_image_2`
            ...
    """

    def __init__(self, args, pos=1):
        """
        """
        Builder.__init__(self)

        self.s2qlst = {pos: []}

        pinfo("Loading image list ")
        pinfo(".")
        trainlst = [line.split() for line in loadlist(args.trainlst)]
        pinfo(".")
        vallst = [line.split() for line in loadlist(args.vallst)]
        pinfo(".")
        self.imglst = [line[0] for line in trainlst] + \
            [line[0] for line in vallst]
        pinfo(".")
        pinfo(" Done!\n")

        pinfo("\tGenerating DB/query list ")
        pinfo(".")
        self.dblst = np.array(range(len(trainlst)))
        pinfo(".")
        self.qrylst = np.array(range(len(vallst)))
        pinfo(".")
        pinfo(" Done!\n")

        pinfo("\tGenerating label list ")
        pinfo(".")
        self.dlblst = np.array([int(line[1]) for line in trainlst])
        pinfo(".")
        self.qlblst = np.array([int(line[1]) for line in vallst])
        pinfo(".")
        pinfo(" Done!\n")

        pinfo("\tGenerating pos list \n")
        self.poslst = []
        self.amblst = []
        for qid in self.qrylst:
            if qid % 100 == 0:
                pinfo("\r\t\t%d/%d" % (qid, len(self.qrylst)))
            self.poslst.append(self.dblst[self.dlblst == self.qlblst[qid]])
            self.amblst.append([])
        self.s2qlst[pos] = self.poslst
        pinfo("\r\t\t%d/%d\tDone!\n" % (len(self.qrylst), len(self.qrylst)))

    def __del__(self):
        pass

    @classmethod
    def set_parser(cls, parser):
        Builder.set_parser(parser)
        parser.add_argument('trainlst', type=str,
                            help="""training image list""")
        parser.add_argument('vallst', type=str,
                            help="""validation image list""")


DATASETS = {
    'caltech':     Caltech,
    'imgnet2012':  ImageNet2012,
    'ukbench':     UKBench,
    'holidays':    Holidays,
    'parism':      ParisM,
    'paris':       Paris,
    'oxfordm':     OxfordM,
    'oxford':      Oxford}



#!/usr/bin/env python
# coding: utf-8

################################################################
# Dataset Builder ##############################################
################################################################

"""
   File Name: dsinit.py
      Author: Wan Ji
      E-mail: wanji@live.com
  Created on: Tue Mar 18 14:07:56 2014 CST
"""
DESCRIPTION = """
"""

import random
import argparse
import irbench.dataset as irds


def getargs():
    """ Parse program arguments.
    """

    parser = argparse.ArgumentParser(description=DESCRIPTION,
                                     formatter_class=
                                     argparse.RawTextHelpFormatter)
    subparsers = parser.add_subparsers(dest='dataset', help='dataset help')
    for (name, cls) in irds.DATASETS.iteritems():
        ds_parser = subparsers.add_parser(name, help=cls.__doc__)
        cls.set_parser(ds_parser)
    parser.add_argument('seed', type=int, nargs='?', default=0,
                        help="seed for random number generator")

    return parser.parse_args()


def main(args):
    """ Main entry.
    """

    # to ensure that the generated list is consistent every time
    random.seed(args.seed)

    dataset = irds.DATASETS[args.dataset](args)
    dataset.generate(args.trg_dir)

if __name__ == '__main__':
    main(getargs())



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: proto/dataset.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)




DESCRIPTOR = _descriptor.FileDescriptor(
  name='proto/dataset.proto',
  package='irbench',
  serialized_pb='\n\x13proto/dataset.proto\x12\x07irbench\",\n\tSim2Query\x12\x0f\n\x07sim2qry\x18\x01 \x02(\x02\x12\x0e\n\x06imglst\x18\x02 \x03(\x05\"^\n\x05Query\x12\r\n\x05query\x18\x01 \x02(\x05\x12\x0e\n\x06poslst\x18\x02 \x03(\x05\x12\x0e\n\x06\x61mblst\x18\x03 \x03(\x05\x12&\n\nsim2qrylst\x18\x04 \x03(\x0b\x32\x12.irbench.Sim2Query\"M\n\x06Search\x12\x11\n\x05\x64\x62lst\x18\x01 \x03(\x05\x42\x02\x10\x01\x12\x1e\n\x06qrylst\x18\x02 \x03(\x0b\x32\x0e.irbench.Query\x12\x10\n\x08trainlst\x18\x07 \x03(\x05\"H\n\x07\x44\x61taset\x12\x0c\n\x04name\x18\x01 \x02(\t\x12\x0f\n\x07\x63omment\x18\x02 \x02(\t\x12\x0e\n\x06images\x18\x03 \x03(\t\x12\x0e\n\x06labels\x18\x04 \x03(\x05')




_SIM2QUERY = _descriptor.Descriptor(
  name='Sim2Query',
  full_name='irbench.Sim2Query',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sim2qry', full_name='irbench.Sim2Query.sim2qry', index=0,
      number=1, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='imglst', full_name='irbench.Sim2Query.imglst', index=1,
      number=2, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=32,
  serialized_end=76,
)


_QUERY = _descriptor.Descriptor(
  name='Query',
  full_name='irbench.Query',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='query', full_name='irbench.Query.query', index=0,
      number=1, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='poslst', full_name='irbench.Query.poslst', index=1,
      number=2, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='amblst', full_name='irbench.Query.amblst', index=2,
      number=3, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='sim2qrylst', full_name='irbench.Query.sim2qrylst', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=78,
  serialized_end=172,
)


_SEARCH = _descriptor.Descriptor(
  name='Search',
  full_name='irbench.Search',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='dblst', full_name='irbench.Search.dblst', index=0,
      number=1, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), '\020\001')),
    _descriptor.FieldDescriptor(
      name='qrylst', full_name='irbench.Search.qrylst', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='trainlst', full_name='irbench.Search.trainlst', index=2,
      number=7, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=174,
  serialized_end=251,
)


_DATASET = _descriptor.Descriptor(
  name='Dataset',
  full_name='irbench.Dataset',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='irbench.Dataset.name', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='comment', full_name='irbench.Dataset.comment', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='images', full_name='irbench.Dataset.images', index=2,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='labels', full_name='irbench.Dataset.labels', index=3,
      number=4, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=253,
  serialized_end=325,
)

_QUERY.fields_by_name['sim2qrylst'].message_type = _SIM2QUERY
_SEARCH.fields_by_name['qrylst'].message_type = _QUERY
DESCRIPTOR.message_types_by_name['Sim2Query'] = _SIM2QUERY
DESCRIPTOR.message_types_by_name['Query'] = _QUERY
DESCRIPTOR.message_types_by_name['Search'] = _SEARCH
DESCRIPTOR.message_types_by_name['Dataset'] = _DATASET

class Sim2Query(_message.Message):
  __metaclass__ = _reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _SIM2QUERY

  # @@protoc_insertion_point(class_scope:irbench.Sim2Query)

class Query(_message.Message):
  __metaclass__ = _reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _QUERY

  # @@protoc_insertion_point(class_scope:irbench.Query)

class Search(_message.Message):
  __metaclass__ = _reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _SEARCH

  # @@protoc_insertion_point(class_scope:irbench.Search)

class Dataset(_message.Message):
  __metaclass__ = _reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _DATASET

  # @@protoc_insertion_point(class_scope:irbench.Dataset)


_SEARCH.fields_by_name['dblst'].has_options = True
_SEARCH.fields_by_name['dblst']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), '\020\001')
# @@protoc_insertion_point(module_scope)



#!/usr/bin/env python
# coding: utf-8

#########################################################################
#########################################################################

"""
   File Name: bench.py
      Author: Wan Ji
      E-mail: wanji@live.com
  Created on: Sat Mar 15 12:05:21 2014 CST
"""
DESCRIPTION = """
Image Retrieval Benchmark.

++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+ NOTE:
+     If -t[--tmpdir] is not specified, -ow[--overwrite] will have no effect.
+     Otherwise, if -ow specified, the temporal files will be overwrited;
+                if -ow not specified, the programm will read from existing
+                temporal files.
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
"""

import os
import argparse
import logging

import numpy as np
import scipy.io as spio

from irbench import index
from irbench import evaluate
from irbench.utils import runcmd, load_lst_of_lst

IMAGEFILE = 'image.lst'
LABELFILE = 'label.lst'
SIM_FILE = 'similarity.lst'
TMPF_RANKED = 'ranked'
TMPF_LINIDX = 'linidx'
TMPF_RESULT = 'result'
TMPF_DISTANCE = 'distance'


def getargs():
    """ Parse program arguments.
    """

    parser = argparse.ArgumentParser(description=DESCRIPTION,
                                     formatter_class=
                                     argparse.ArgumentDefaultsHelpFormatter)

    # database related options
    parser.add_argument('--dbpath', type=str, required=True,
                        help='path of database features')
    parser.add_argument('--dbitems', type=str,
                        help='list of database items')

    # query related
    parser.add_argument('--qrypath', type=str, required=True,
                        help='path of query features')
    parser.add_argument('--qryitems', type=str,
                        help='list of query items')

    # distraction related
    parser.add_argument('--dispath', type=str, nargs='?', default=None,
                        help='path of distracting features')
    parser.add_argument('--disitems', type=str, nargs='?', default=None,
                        help='list of distracting items')

    # measurement
    parser.add_argument('--preck', type=int, nargs='+', default=None,
                        help="precesion@k, should be integers")
    parser.add_argument('--recallk', type=int, nargs='+', default=None,
                        help="recall@k, should be integers")
    parser.add_argument('--ndcgk', type=int, nargs='+', default=None,
                        help="ndcg@k, should be integers")
    parser.add_argument('--map', const=True, action='store_const',
                        default=False, help='overwrite the temporal files')
    parser.add_argument('--poslst', type=str, default=None, nargs='?',
                        help="positive list")
    parser.add_argument('--amblst', type=str, default=None, nargs='?',
                        help="ambiguity list")
    parser.add_argument('--simlst', type=str, default=None, nargs='?',
                        help="similarity list")
    parser.add_argument('-m', '--metric', type=str, default=None,
                        help='metric file (mat file containing `M`)')

    add_other_options(parser)

    return parser.parse_args()


def add_other_options(parser):
    """
    Add other options to the parser
    """
    # other options
    parser.add_argument('-k', '--topk', type=int, default=20000,
                        help='number of returned results')
    parser.add_argument('-d', '--distance', type=str, default="cosine",
                        help='type distance metric')
    parser.add_argument('-i', '--indexer', type=str, default="linear",
                        help='type of indexer')
    parser.add_argument('-r', '--ranked', type=str, default=None,
                        help="""evaluate on ranked files directly
                             (skip indexing and querying)""")
    parser.add_argument('-t', '--tmpdir', type=str,
                        help='dir for saving temporal files')
    parser.add_argument('-ow', '--overwrite', const=True, action='store_const',
                        default=False, help='overwrite the temporal files')
    parser.add_argument('--log', type=str, nargs="?", default="INFO",
                        help='overwrite the temporal files')


def rankedload(rankedpath):
    """ Load rank from pickle file
    """

    if os.path.isdir(rankedpath):
        ranklst = sorted([int(fname.split(".")[0])
                          for fname in os.listdir(rankedpath)])
        v_rank = []
        for rid in ranklst:
            v_rank.append(spio.loadmat(
                os.path.join(rankedpath, "%d.mat" % rid))['search_index'])
        ranked = np.hstack(tuple(v_rank))
    else:
        ranked = spio.loadmat(rankedpath)['search_index']

    return ranked


def rankeddump(ranked, rankedpath, batchsize=1000):
    """ Dump rank to pickle file
    """
    if ranked.shape[1] > batchsize:
        runcmd("mkdir -p %s" % rankedpath)
        for rid in range(0, ranked.shape[1], batchsize):
            rest = min(ranked.shape[1] - rid, batchsize)
            spio.savemat(os.path.join(rankedpath, "%d.mat" % rid),
                         {'search_index': ranked[:, rid:rid+rest]})
    else:
        spio.savemat(rankedpath, {'search_index': ranked})


def ranking(args):
    """
    Ranking
    """
    logging.info("################################")
    logging.info("# Building indices #############")
    logging.info("################################")
    if args.metric:
        indexer = index.INDEXER_TABLE[args.indexer](
            distype=args.distance,
            metric=index.Indexer.load_metric(args.metric))
    else:
        indexer = index.INDEXER_TABLE[args.indexer](
            distype=args.distance,
            metric=None)
    logging.info("Indexer created.")

    logging.info("Adding database items - ^^^^^")
    dblst = np.loadtxt(args.dbitems, np.int)

    # logging.info("\t********************************")
    # dblst = dblst[:25600]
    # logging.info("\t* WARNING: FOR DEBUG ONLY! *****")
    # logging.info("\t********************************")

    (startnum, nitems) = indexer.add(args.dbpath, dblst)
    logging.info("Adding database items - $$$$$")
    logging.info("\t(%d items added, %d in total)" %
                 (nitems - startnum, nitems))

    # create idmap
    idmap = dblst

    if None != args.dispath:
        logging.info("Adding distracting items - ^^^^^")
        dislst = np.loadtxt(args.disitems, np.int)
        (startnum, nitems) = indexer.add(args.dispath, dislst)
        logging.info("Adding distracting items - $$$$$")
        logging.info("\t(%d items added, %d in total)" %
                     (nitems - startnum, nitems))

        # mapping distracting list to -1
        idmap = np.hstack((idmap, [-1 for i in dislst]))

    indexer.adjust()

    logging.info("################################")
    logging.info("# Querying #####################")
    logging.info("################################")
    logging.info("Querying - ^^^^^")
    qrylst = np.loadtxt(args.qryitems, np.int)
    ranked = indexer.query(args.qrypath, qrylst, args.topk)
    logging.info("Querying - $$$$$")

    """
    Checking whether the query list is included in the database list.
    **  If it is True, the query item itself should be excluded from the
        ranked list.
    """
    b_exclude = True
    if args.dbpath != args.qrypath:
        # query feature and db feature are not the same,
        # no need for excluding query from results
        b_exclude = False
    else:
        for qid in qrylst:
            # query is not included in db,
            # no need for excluding query from results
            if qid not in dblst:
                b_exclude = False
                break

    if False and not b_exclude:
        logging.warning("Excluding queries from results - ^^^^^")
        ex_ranked = np.ndarray((ranked.shape[0], ranked.shape[1] - 1),
                               ranked.dtype)
        for rid in range(ranked.shape[0]):
            ex_ranked[rid, :] = ranked[rid, ranked[rid, :] != qrylst[rid]]
        ranked = ex_ranked
        logging.warning("Excluding queries from results - $$$$$")

    logging.info("Results mapping - ^^^^^")
    ranked = idmap[ranked]
    logging.info("Results mapping - $$$$$")

    # saving retrieval results
    if None != args.tmpdir:
        dumpath = os.path.sep.join([args.tmpdir, TMPF_RANKED])
        logging.info("Dumping to %s - ^^^^^" % dumpath)
        rankeddump(ranked, dumpath)
        logging.info("Dumping to %s - $$$$$" % dumpath)

    return ranked


def evaluating(args, ranked):
    """
    Evaluating
    """
    logging.info("################################")
    logging.info("# Evaluating ###################")
    logging.info("################################")
    logging.info("Evaluating - ^^^^^")

    eval_res = {}
    evaluator = evaluate.Evaluator()

    # simlst = None
    # if args.ndcgk:
    #     logging.info("\tLoading similarity to query - ^^^^^")
    #     (s2qdir, s2qpre) = os.path.split(dataset.Builder.S2Q_LIST)
    #     s2qdir = os.path.sep.join([args.datadir, s2qdir])
    #     s2qpre += '.'
    #     simlst = [{} for qid in range(ranked.shape[0])]
    #     for fname in os.listdir(s2qdir):
    #         # print s2qpre, fname
    #         if (s2qpre in fname) and (fname[-2:] != "_1"):
    #             simval = float(fname.split(s2qpre)[-1])
    #             simpath = os.path.sep.join([s2qdir, fname])
    #             v_lst = load_lst_of_lst(simpath, int)
    #             for qid in range(ranked.shape[0]):
    #                 simlst[qid][simval] = v_lst[qid]
    #     logging.info("\tLoading similarity to query - $$$$$")

    poslst = None
    amblst = None
    if args.map or args.preck or args.recallk:
        logging.info("\tLoading pos/amb list - ^^^^^")
        poslst = load_lst_of_lst(args.poslst, int)
        amblst = load_lst_of_lst(args.amblst, int)
        logging.info("\tLoading pos/amb list - $$$$$")

    if args.ndcgk:
        logging.warning("\tThis function need refinment!")
    #     eval_res['ndcgk'] = evaluator.eval_rank_ndcgk(
    #         ranked, simlst, {'Ks': args.ndcgk})

    if args.map:
        eval_res['map'] = evaluator.eval_rank_map(
            ranked, poslst, amblst, {})

    if args.preck:
        eval_res['preck'] = evaluator.eval_rank_preck(
            ranked, poslst, amblst, {'Ks': args.preck})

    if args.recallk:
        eval_res['recallk'] = evaluator.eval_rank_recallk(
            ranked, poslst, amblst, {'Ks': args.recallk})
    logging.info("Evaluating - $$$$$")

    return eval_res


def main(args):
    """ Main entry.
    """
    print args

    # checking temporal directory
    if None != args.tmpdir:
        if os.path.exists(args.tmpdir):
            if args.overwrite:
                logging.warning("Overwrite %s" % args.tmpdir)
            else:
                logging.fatal("%s already exists!" % args.tmpdir)
        runcmd("mkdir -p %s" % args.tmpdir)

    ################################
    # ranking
    ################################
    if None == args.ranked:
        ranked = ranking(args)
    else:
        ranked = rankedload(args.ranked)

    ################################
    # evaluating
    ################################
    eval_res = evaluating(args, ranked)

    ################################
    # generating reports
    ################################
    evaluate.res_print(eval_res)

    # the following code will be removed in future
    try:
        print "++++++++++++++++++++++++++++++++"
        print "****  %f\t%s\t%s" % \
            (eval_res['map'].mean(0),
            '\t'.join(["%f" % x for x in eval_res['preck'].mean(0)]),
            '\t'.join(["%f" % x for x in eval_res['recallk'].mean(0)]))
        print "^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^"
    except:
        pass

    ################################
    # saving evaluation results
    ################################
    if None != args.tmpdir:
        spio.savemat(os.path.sep.join([args.tmpdir, TMPF_RESULT]), eval_res)


if __name__ == '__main__':
    args = getargs()
    numeric_level = getattr(logging, args.log.upper(), None)
    if not isinstance(numeric_level, int):
        raise ValueError('Invalid log level: %s' % args.log)
    logging.basicConfig(format='%(asctime)s - %(levelname)s - %(message)s',
                        level=numeric_level)
    main(args)



#!/usr/bin/env python
# coding: utf-8
"""
   File Name: setup.py
      Author: Wan Ji
      E-mail: wanji@live.com
  Created on: Tue 18 Mar 2014 12:03:18 PM CST
 Description:
"""

import os

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

try:
    from pypandoc import convert
    read_md = lambda f: convert(f, 'rst')
except ImportError:
    print "warning: pypandoc module not found, DONOT convert Markdown to RST"
    read_md = lambda f: open(f, 'r').read()


def runcmd(cmd):
    """ Run command.
    """
    os.system(cmd)


setup(name='irbench',
      version='0.0.56',
      author='WAN Ji',
      author_email='wanji@live.com',
      package_dir={'irbench': 'src'},
      packages=['irbench', 'irbench.proto'],
      scripts=['bin/bench.py',
               'bin/dsinit.py',
               'tools/bow2sparsedir.py',
               'tools/bow2sparse.py'],
      url='https://github.com/wanji/irbench',
      # license='LICENSE.txt',
      description='Image Retrieval Benchmark.',

      long_description=read_md("DESC.md"),
      install_requires=[
          "numpy      >= 1.6.0",
          "scipy      >= 0.9.0",
          "bottleneck >= 0.8.0",
          "bitmap     >= 0.0.1",
          "protobuf   >= 2.4.0",
      ],
      )



#!/usr/bin/env python
# coding: utf-8

#########################################################################
#########################################################################

"""
   File Name: feature.py
      Author: Wan Ji
      E-mail: wanji@live.com
  Created on: Fri Mar 14 17:26:08 2014 CST
"""
DESCRIPTION = """
"""

import os
import logging

import numpy as np
import scipy.io as spio
from scipy.sparse import csr_matrix

from utils import loadsvm


class FeatLoader(object):
    """ load feature from dir
    """
    BATCHTEMP = '%s/feat_%d'

    def __init__(self, featpath, featlst=None):
        """ Initialize the loader
        """
        # indices of features to be loaded
        self.featlst = featlst
        # number of features in the processed batches
        self.n_procfeat = 0
        # next batch to be processed
        self.next_batch = 0

        # init the list of batches
        self.pathlst = []
        if os.path.isdir(featpath):
            fnames = os.listdir(featpath)
            fidxs = [int(batf.split("_")[1].split(".")[0]) for batf in fnames]
            self.pathlst = [os.path.sep.join([featpath, fnames[idx]]) for
                            idx in np.array(fidxs).argsort()]
        else:
            self.pathlst.append(featpath)
        print self.pathlst

    @classmethod
    def loadfeat(cls, featpath):
        """ Load feature from file.
        """
        featname = os.path.split(featpath)[1]
        try:
            logging.debug("Load `%s` as NPY - ^^^^^" % featname)
            prel = np.load(featpath)
            # parse as sparse matrix
            if len(prel) == 2:
                feat = csr_matrix((prel[0], prel[1]))
            # parse as dense matrix
            else:
                feat = prel
            logging.debug("Load `%s` as NPY - $$$$$" % featname)
        except IOError as err:
            logging.debug("%s" % err.message)
            logging.debug("Load `%s` as NPY - failed" % featname)
            try:
                logging.debug("Load `%s` as MAT - ^^^^^" % featname)
                feat = spio.loadmat(featpath)['feat']
                logging.debug("Load `%s` as MAT - $$$$$" % featname)
            except ValueError as err:
                logging.debug("%s" % err.message)
                logging.debug("Load `%s` as MAT - failed" % featname)

                logging.debug("Load `%s` as SVM - ^^^^^" % featname)
                feat = loadsvm(featpath)[1]
                logging.debug("Load `%s` as SVM - $$$$$" % featname)
        return feat

    def load_next(self):
        """ Load next batch
        """

        # no more batches
        if self.next_batch >= len(self.pathlst):
            return None, None
        # no more list items
        if self.featlst is not None and self.n_procfeat > self.featlst.max():
            return None, None

        # if batch file not exist, an exception will be raised
        batchpath = self.pathlst[self.next_batch]
        feat = FeatLoader.loadfeat(batchpath)

        self.next_batch += 1

        n_procfeat_next = self.n_procfeat + feat.shape[0]
        # get the list of relative index in current batch
        idxlst = None
        if self.featlst is not None:
            idxlst = [idx for idx in self.featlst if
                      (idx < n_procfeat_next and idx >= self.n_procfeat)]
            feat = feat[[idx - self.n_procfeat for idx in idxlst], :]
        self.n_procfeat = n_procfeat_next
        return feat, idxlst
